#!/usr/bin/env python
#
# Copyright (C) 2018 Lenovo Corporation
#
# Time: 2018/03/07
# Description:  class for nagios plugin installation
# Compatible python2.7 and python3.5
# use distutils.log to replace print

from os import path, sep, mkdir, walk, remove, access, W_OK
from argparse import ArgumentParser
from shutil import copy
import subprocess
from filecmp import cmp 
from time import sleep
import json


# this code just for compatible python2.7 and python3.5
from distutils.log import Log
# Init log level to infor
log = Log(2)
printf = log.info

def check_python_library(modulename):
	import imp
	try:
		imp.find_module(modulename)
		return True
	except ImportError:
		return False

modulelist = ['cryptography']
printf ("\n  Checking python libraries dependency: %s" % ','.join(modulelist))
for index, module in enumerate(modulelist):
	ret = check_python_library(module)
	if not ret:
		printf ("\n  Python module '%s' is not available, the installation scripts will fail, please install it first" % module)
		printf ("\n  For example: pip install %s" % module)
		exit(0)


from sys import version as python_version 
from sys import version_info as python_info
if python_info < (3, 0):
    lx_input = raw_input
else:
    lx_input = input


# Import function from my module
from nagios_cmk_base import NagiosCmkBase
from active_discovery import ActiveDiscovery
from common_lib import select_y_n
from parser_discovery_cfg import ParserDiscoveryCfg
from common_lib import get_group_dict_info
from common_lib import encrypt
from common_lib import decrypt


NAGIOS_CFG_NAME = "lenovo_server.cfg"


g_contact_info = """
define contact{
    contact_name     %s            
    use              generic-contact 
    alias            %s 
    email            %s 
}

"""

g_contact_group = """
define contactgroup{
    contactgroup_name       lnv_admins
    alias                   lenovo group
    members                 %s
}

"""

host_note = """
###############################################################################
###############################################################################
#
# HOST DEFINITIONS
#
###############################################################################
###############################################################################

# Define hosts for the Lenovo XCC machines we'll be monitoring
# Change the host_name, alias, and address to fit your situation
"""

group_note = """
###############################################################################
###############################################################################
#
# HOST GROUP DEFINITIONS
#
###############################################################################
###############################################################################

# Define hostgroups for Lenovo XCC machines
# All hosts that use the linux-server template will automatically be a member of this group
"""

service_note = """
###############################################################################
###############################################################################
#
# SERVICE DEFINITIONS
#
###############################################################################
###############################################################################

# Create services for monitoring
"""

host_group_tmp = """
define hostgroup{
	hostgroup_name    %s  ; The name of the hostgroup
	alias             %s  ; Long name of the group
	members           %s  ; Members in this group
}
"""

service_tmp = """
define service{
	use                 generic-service
	hostgroup_name      %s
	service_description Check %s 
	check_command       %s
	contact_groups      lnv_admins
}
"""

alert_service_tmp = """
define service{
	use                 generic-service
	hostgroup_name      %s
	active_checks_enabled           0
	passive_checks_enabled          1
	service_description %s 
	check_command       check-host-alive
	contact_groups      lnv_admins
	max_check_attempts              1
	retry_check_interval            1
	is_volatile                     1
	#obsess_over_service            0
}
"""


host_tmp = """
define host{
    use             linux-server    ; Inherit default values from a template
    host_name       %s ; The name we're giving to this host
    alias           %s ; A longer name associated with the host
    address         %s  ; IP address of the host
}
"""

command_tmp = """
################################################################################
#
# LENOVO SERVICE CHECK COMMANDS
#
# These are some example service check commands.  They may or may not work on
# your system, as they must be modified for your plugins.  See the HTML 
# documentation on the plugins for examples of how to configure command definitions.
#
################################################################################

define command{
        command_name    check_system_authPriv
        command_line    $USER1$/lenovo_plugins/check_lenovo_xcc.py -H "$HOSTADDRESS$" --snmp_username $ARG1$ --snmp_security_level authPriv --snmp_aprotocol $ARG2$ --snmp_apassword $ARG3$ --atag $ARG4$  --snmp_pprotocol $ARG5$ --snmp_ppassword $ARG6$ --ptag $ARG7$ --mode $ARG8$ 
        }

define command{
        command_name    check_system_authNoPriv
        command_line    $USER1$/lenovo_plugins/check_lenovo_xcc.py -H "$HOSTADDRESS$" --snmp_username $ARG1$ --snmp_security_level authNoPriv --snmp_aprotocol $ARG2$ --snmp_apassword $ARG3$ --atag $ARG4$ --mode $ARG5$ 
        }

define command{
        command_name    check_system_noAuthNoPriv
        command_line    $USER1$/lenovo_plugins/check_lenovo_xcc.py -H "$HOSTADDRESS$" --snmp_username $ARG1$ --snmp_security_level noAuthNoPriv --mode $ARG2$ 
        }

"""


class NagiosPlugin(NagiosCmkBase):

	def __init__(self, local_path = "", config_file = "", importfile = "", actv_dscry = False, pstv_dscry = False): 
		self.site_path = self.__get_path_nagios()
		self.local_path = local_path
		self.config_file = config_file
		self.importfile = importfile
		self.actv_dscry = actv_dscry
		self.pstv_dscry = pstv_dscry


	################################################################
	#
	# Get the path of nagios 
	#
	################################################################
	# Get the path of nagios
	def __get_path_nagios(self, depth=1):

		nagios_path = lx_input("\nProvide the installed location of Nagios Core\n    (Press ENTER to continue with the default location: '/usr/local/nagios/'):")

		if nagios_path:
			if not path.isdir(nagios_path):
				printf ("    %s is not a direction" % nagios_path)
				if depth >= 3:
					printf ("    Nagios path is /usr/local/nagios/\n")
					return "/usr/local/nagios/"
				nagios_path = self.__get_path_nagios(depth+1)
			else:
				printf ("    Nagios path is %s\n" % nagios_path)
		else:
			nagios_path = "/usr/local/nagios/"
			printf ("    Nagios path is %s\n" % nagios_path)

		if nagios_path.strip().endswith('/'):
			nagios_path = nagios_path.strip()
		else:
			nagios_path = nagios_path.strip() + '/'
		
		return nagios_path



	################################################################
	#
	# Check the dependences of Lenovo_Auto_install.py
	#
	###############################################################
	def __check_dependence(self):
		printf ("\nChecking Requires...")
		sleep(1)
		ret1 = self.__check_python_version()
		ret2 = self.__check_net_snmp_version()
		if not ret1:
			printf ("\n  Python is not available, this automatic installation tool will fail\n")
		if not ret2:
			printf ("\n  Net-SNMP is not available, active discovery will fail\n")
		else:
			printf ("\n  ")


	def __check_python_version(self):
		version_str = python_version.split(" ")[0]
		if '2.7' in version_str or '3.5' in version_str:
			printf ("    Python\tversion: v%s\tOK" % version_str)
			return True
		else:
			printf ("    Python\tnot available")
			return False


	def __check_net_snmp_version(self):
		cmd = "snmpwalk --version"
		run_watch = subprocess.Popen(cmd.split(), stdout = subprocess.PIPE, stderr = subprocess.PIPE, universal_newlines=True)
		ret, err_msg = run_watch.communicate()
		ret = ret.strip() + err_msg.strip()
		net_snmp_version = ret.strip().split(" ")[-1]
		if net_snmp_version:
			printf ("    Net-SNMP\tversion: v%s\tOK" % net_snmp_version)
			return True
		else:
			printf ("    Net-SNMP\tnot available")
			return False


	######################################################
	#
	# Install nagios config file which created in ./etc/
	#
	#####################################################
	# install config file to nagios
	def __install_etc_file(self, nagios_path):
		cfg_list = self.__get_etc_list()
		self.__install_etc(nagios_path, cfg_list)


	# get etc list, all the .cfg file in tools/etc/
	def __get_etc_list(self):
		etc_file_list = []
		cfg_store_path = path.join(self.local_path, 'tools/etc')
		for root, dirs, files in walk(cfg_store_path):
			for name in files:
				if name.endswith('.cfg'):
					etc_file_name = path.join(root, name)
					etc_file_list.append(etc_file_name)
		return etc_file_list


	# install config file to nagios/etc
	def __install_etc(self, nagios_path, cfg_list):
		printf ("\nInstalling the Nagios config file...")
		cfg_path = path.join(nagios_path, "etc/lenovo_cfg")
		if path.isdir(cfg_path):
			pass
		else:
			try:
				mkdir(cfg_path)
			except:
				printf ("    Failed in installing Nagios config file! ")
				return False

		# install config file to etc/lenovo_cfg
		for cfg_file in cfg_list:
			tmp_path = path.join(cfg_path, path.basename(cfg_file))

			if not path.exists(tmp_path):
				copy(cfg_file, tmp_path)
				printf ("  Copy file %s to %s " % (cfg_file, tmp_path))
			else:
				if cmp(cfg_file, tmp_path):
					printf ("  %s is same with %s " % (cfg_file, tmp_path))
				else:
					select_val = select_y_n("  File %s exists, would you want to replace it? (y/n)(Blank is y)" % tmp_path)
					if select_val.lower() == "y":
						copy(cfg_file, tmp_path)
						printf ("  Copy file %s to %s " % (cfg_file, tmp_path))
					else:
						printf ("  This file has been stored at %s, you can hand it manually." % cfg_file)


	def __install_plugin(self):
		"""	Install plugin 'check_lenovo_xcc.py' in to Nagios

		Copy script 'check_lenovo_xcc.py' into $(Nagios_path)/libexec/lenovo_plugins/ 

		Args:
			local_path: the path of work dir
			site_path: the path of nagios 
		"""
		local_path = self.local_path
		site_path = self.site_path

		printf("Installing Plugin-In 'check_lenovo_xcc.py' ...")

		src_plugin = local_path + "tools/libexec/check_lenovo_xcc.py"
		dst_path = site_path + "libexec/lenovo_plugins/"
		if path.isdir(dst_path):
			pass
		else:
			try:
				print dst_path
				mkdir(dst_path)
			except:
				printf ("    Failed in installing plugin")
				return
		dst_plugin = site_path + "libexec/lenovo_plugins/check_lenovo_xcc.py"

		if not path.exists(dst_plugin):
			copy(src_plugin, dst_plugin)
			printf ("    Copy file %s to %s" % (src_plugin, dst_plugin))
		else:
			if cmp(src_plugin, dst_plugin):
				printf ("    'check_lenovo_xcc.py' has been installed")
			else:
				select_val = select_y_n("    File %s exists, would you want to replace it? (y/n) (Blank is y)" % dst_plugin)
				if select_val.lower() == "y":
					copy(src_plugin, dst_plugin)
					printf ("    Copy file %s to %s" % (src_plugin, dst_plugin))
				else:
					printf ("    File check_lenovo_xcc.py is saved in %s, you can hand it manually.", local_path + "tools/libexec/")

		# Copy file traphandle.py to libexec/lenovo_plugins/
		printf("Installing Plugin-In 'traphandle.py' ...")
		src_traphandle = local_path + "tools/libexec/traphandle.py"
		dst_traphandle = site_path + "libexec/lenovo_plugins/"
		copy(src_traphandle, dst_traphandle)
		printf ("    Copy file %s to %s" % (src_traphandle, dst_traphandle))

		# Copy file alerthandle.py to libexec/lenovo_plugins/
		printf("Installing Plugin-In 'alerthandle.py' ...")
		src_alerthandle = local_path + "tools/libexec/alerthandle.py"
		dst_alerthandle = site_path + "libexec/lenovo_plugins/"
		copy(src_alerthandle, dst_alerthandle)
		printf ("    Copy file %s to %s" % (src_alerthandle, dst_alerthandle))
			



	####################################################
	#
	# Set Email contact group
	#
	###################################################
	def __set_contacts_group(self):
		nagios_contact_file = self.site_path + "etc/lenovo_cfg/lenovo_contact.cfg"
		contact_file = self.local_path + "tools/etc/lenovo_contact.cfg"
		if not path.isfile(nagios_contact_file):
			mail_list_str = lx_input("\nPlease set a comma-separated Email address list:\n")
			mail_list = mail_list_str.split(",")
			all_mail_str = ""
			contact_str = ""
			contact_group_str = ""
			for mail in mail_list:
				contact_str += g_contact_info % (mail, mail, mail)
				all_mail_str += mail + ","
			contact_group_str = (g_contact_group % (all_mail_str))
			try: 
				fh = open(contact_file, "w+")
				fh.write(contact_str + contact_group_str)
				fh.close()
				print("Succeed to set Email contacts")
			except:
				print("Failed in writing configuration file")
				

	####################################################
	#
	# Discovery and create config file for nagios
	#
	###################################################
	def __get_actv_dscvr_dict(self, cfg_file):
		"""
		Get active discovery dict
		"""
		if cfg_file:
			if not path.isfile(cfg_file):
				printf ("'%s' which specified by parameter '-f' is not a file!" % cfg_file)
				cfg_file = self._get_discovery_cfg_file(2)
		else:
			cfg_file = self._get_discovery_cfg_file()
			
		sleep(1)
		self.config_file = cfg_file

		actv_dict = self.get_discovery_dict(self.config_file, True)

		if actv_dict:
			pass
		else:
			print("    No host be added in Nagios in this discovery")

		return actv_dict

	def __get_import_dict(self, import_file):
		"""
		Get import dict
		"""
		import_dict = dict()

		if import_file:
			if not path.isfile(import_file):
				printf ("'%s' which specified by parameter '-i' is not a file!" % import_file)
				return import_dict
			else:
				sleep(1)
				import_dict = self.get_import_dict(import_file)

				if import_dict:
					pass
				else:
					print("    No host be added into Nagios in this import")
				return import_dict
		else:
			return import_dict
			
		

	# Create config file through passive discovery
	def __passive_discovery(self, passive_cfg, nagios_cfg, nagios_path):
		"""
		Passive discovery
		"""
		if passive_cfg:
			if not path.isfile(passive_cfg):
				printf ("'%s' which specified by parameter '-b' is not a file!" % passive_cfg)
				passive_cfg = self._get_discovery_cfg_file(2,"passive_temp.cfg")
		else:
			passive_cfg = self._get_discovery_cfg_file(1, "passive_temp.cfg")

		sleep(1)
		self.passive_discover(passive_cfg, nagios_cfg, nagios_path)



	##################################################
	#
	# add cfg file path(like:cfg_dir=/usr/local/nagios/etc/lenovo_cfg) into nagios.cfg
	#
	#################################################
	def __modify_nagios_cfg(self, nagios_path):
		nagios_cfg = path.join(nagios_path, "etc/nagios.cfg")
		lenovo_cfg_path = path.join(nagios_path, "etc/lenovo_cfg/")
		file_data = ""
		cfg_dir_before = ""
		cfg_dir_after = ""
		find_cfg_dir = False
		cfg_dir_exist = False
		all_data = ""
		with open(nagios_cfg, "r") as f:
			for line in f:
				if "cfg_dir=" in line and find_cfg_dir == False:
					cfg_dir_before = file_data
					file_data = ""
					find_cfg_dir = True
				if lenovo_cfg_path in line:
					cfg_dir_exist = True
				file_data += line
			cfg_dir_after = file_data
		
		# if find ${nagios_path}/etc/lenovo_cfg in nagios.cfg
		if cfg_dir_exist:
			all_data = cfg_dir_before + cfg_dir_after
		else:
			# if find cfg_dir, add cfg_dir=xxx before this line
			if find_cfg_dir:
				all_data = "%s\ncfg_dir=%s\n%s" % (cfg_dir_before, lenovo_cfg_path, cfg_dir_after) 
			else:
				all_data = "%s%s\ncfg_dir=%s\n" % (cfg_dir_before, cfg_dir_after, lenovo_cfg_path)
		
		with open(nagios_cfg, "w") as f:
			f.write(all_data)



	#######################################################
	#
	# verify the config file create by this tool 
	#
	######################################################
	# verify the nagios.cfg file
	def __verify_cfg(self, nagios_path):
		cmd = "%sbin/nagios -v %setc/nagios.cfg" % (nagios_path, nagios_path)
		printf ("\nChecking nagios.cfg with command: %s " % cmd)
		sleep(2)
		run_watch = subprocess.Popen(cmd.split(), stdout = subprocess.PIPE, stderr = subprocess.PIPE, universal_newlines=True)
		ret, err_msg = run_watch.communicate()
		printf ("%s" % ret+err_msg)



	#######################################################
	#
	# Other functions
	#
	######################################################

	# Check the privileges
	def _check_privileges(self, nagios_path):
		etc_path = path.join(nagios_path, 'etc/')
		libexec_path = path.join(nagios_path, 'libexec/')
		etc_can_write = False
		libexec_can_write = False

		etc_can_write = access(etc_path, W_OK)
		libexec_can_write = access(libexec_path, W_OK)

		if etc_can_write and libexec_can_write:
			return True
		else:
			return False



	def passive_discover(self, discover_cfg, nagios_cfg, nagios_path):
		"""
		Set passive discovery environment
		"""
		trap_userinfo = self.__get_userinfo4trapd(discover_cfg, nagios_cfg, nagios_path)
		snmp_trapd_file = self.__get_snmptrapd_path()
		self.__add_useinfo2file(trap_userinfo, snmp_trapd_file)
		self.__start_snmptrapd(snmp_trapd_file)


	def __get_userinfo4trapd(self, discovery_cfg, nagios_cfg, nagios_path):
		"""
		Get userinfor which will add into snmptrapd.cfg
		e.g.:
			createUser -e 0x123456 SNMPUSER SHA PASSW0RD DES PASSW0RD
			authuser log,execute,net SNMPUSER
		"""

		parser_cfg = ParserDiscoveryCfg()
		passive_raw_group_dict = parser_cfg.get_raw_group_dict(discovery_cfg, False)

		for group, group_data in passive_raw_group_dict.items():
			group_data["apassword"], group_data["atag"] = encrypt(group_data["apassword"])
			group_data["ppassword"], group_data["ptag"] = encrypt(group_data["ppassword"])
		passive_data = json.dumps(passive_raw_group_dict)
		with open("%setc/lenovo_cfg/passive.data" % nagios_path, 'w') as f:
			f.write(passive_data)


		print("Setting snmptrapd via passive discovery config file")

		snmp_trap_str = "\n"
		for group in passive_raw_group_dict:
			print("    Setting group [%s]" % group)

			ip_list, engineid_list, alias, user, authlevel, snmp_aprotocol, apassword, atag, snmp_pprotocol, ppassword, ptag, services_list = get_group_dict_info(passive_raw_group_dict[group])
			if engineid_list:
				print("    Engineid:\n        %s" % str(engineid_list))
			else:
				continue

			tmp_str = ""
			
			if authlevel == 'authPriv':
				if atag and ptag:
					apassword = decrypt(apassword, atag)
					ppassword = decrypt(ppassword, ptag)
				tmp_str = " %s %s %s %s %s \n" % (user, snmp_aprotocol, apassword, snmp_pprotocol, ppassword) 
			elif authlevel == 'authNoPriv':
				if atag:
					apassword = decrypt(apassword, atag)
				tmp_str = " %s %s %s \n" % (user, snmp_aprotocol, apassword) 
			elif authlevel == 'noAuthNoPriv':
				tmp_str = " %s \n" % (user) 

			print("    User: %s\n" % user)

			for engineid in engineid_list:
				snmp_trap_str += "createUser -e %s %s" % (engineid, tmp_str)
			snmp_trap_str += "authuser log,execute,net %s \n" % user
		
		## Lenovo-XCC-ALERT-MIB : OID for Power-On 1.3.6.1.4.1.19046.11.1.158.5.0.24 
		snmp_trap_str += "traphandle 1.3.6.1.4.1.19046.11.1.158.5.0.24 %s/libexec/lenovo_plugins/traphandle.py -d %s -g %s \n" % (nagios_path, "%setc/lenovo_cfg/passive.data" % nagios_path, nagios_cfg)

		## Add snmptrap Alert config
		## Lenovo-XCC-ALERT-MIB : Host System UUID(Universal Unique ID) 1.3.6.1.4.1.19046.11.1.158.5.*
		snmp_trap_str += "traphandle 1.3.6.1.4.1.19046.11.1.158.5.* %s/libexec/lenovo_plugins/alerthandle.py %s \n" % (nagios_path, nagios_path)

		return snmp_trap_str


	def __add_useinfo2file(self, useinfo, file_name):
		"""
		Add userinfor into snmptrapd.cfg file
		"""
		useinfo_list = [x + "\n" for x in useinfo.split('\n')]
		trapd_file_handle = open(file_name, 'r')
		trapd_list = trapd_file_handle.readlines()
		trapd_file_handle.close()

		need_write_str = ""
		for useinfo_line in useinfo_list:
			if useinfo_line in trapd_list:
				pass
			else:
				need_write_str += useinfo_line

		with open(file_name, 'a+') as f:
			f.write(need_write_str)
		


	def __get_snmptrapd_path(self):
		"""
		Get the path of snmptrapd.conf
		"""
		if path.isfile('/etc/snmp/snmptrapd.conf'):
			return "/etc/snmp/snmptrapd.conf"
		else:
			print "[Warning]: Can not find file /etc/snmp/snmptrapd.conf"
			return ""



	def __start_snmptrapd(self, snmptrapd_path):
		"""
		Restart snmptrapd
		"""
		print("Restart snmptrapd:")
		if "" != snmptrapd_path:
			#cmd = "snmptrapd -On -C -Le -c %s " % snmptrapd_path
			cmd = "service snmptrapd restart "
			run_watch = subprocess.Popen(
				cmd.split(), stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
			ret, err_msg = run_watch.communicate()
			print ret + err_msg
		else:
			print "Can not restart snmptrapd"




	def load_config_file(self, cfg_file):
		"""
		load nagios config file which defined host,services and groups, return XCC dict
		"""
		list_hostgroup = []
		list_service = []
		list_host = []

		if not path.isfile(cfg_file):
			return {} 

		with open(cfg_file) as f:
			for line in f:
				if line.strip() == "" or line.strip()[0] == "#" :
					continue
				elif ';' in line:
					line_tmp = line[:line.find(";")] + "\n"
				else:
					line_tmp = line
	
				# 1. Get the data segment in 'define xxx{' and '}'
				# 2. Set the data_type according the define name "xxx", and get the lists of hostgroup and service dict
				# 3. Get the full dict according the lists of hostgroup and service dict
				# 4. Note: the host list is not used
				if "define host{" in line_tmp or "define service{" in line_tmp or "define hostgroup{" in line_tmp:
					cfg_data_segment = line_tmp
				elif "}" == line_tmp.strip():
					cfg_data_segment += line_tmp
					data_type,dict_info = self.__handle_cfg_data_segment(cfg_data_segment)
					if "hostgroup" == data_type:
						list_hostgroup.append(dict_info)
					elif "service" == data_type:
						list_service.append(dict_info)
					elif "host" == data_type:
						list_host.append(dict_info)
					else:
						pass
				else:
					cfg_data_segment += line_tmp
			
			dict_return = self.__get_dict_via_group_service(list_hostgroup, list_service)
		return dict_return

		
	def __handle_cfg_data_segment(self, cfg_data_segment):
		"""
		according to the config file segment,
		return the data type and a dict for this data
		"""
		data_type = "unknown"
		dict_return = {}
		cfg_infor = cfg_data_segment.split('\n')
		if 0 == len(cfg_infor):
			return data_type, dict_return
		if 'hostgroup' in cfg_infor[0]:
			data_type = 'hostgroup'
		elif 'service' in cfg_infor[0]:
			data_type = 'service'
		elif 'host' in cfg_infor[0]:
			data_type = 'host'
		else:
			return data_type, dict_return

		# Get the dict which key is the string before first space, and others string is value
		# eg. "host test1", the key is 'host' and the value is 'test1'
		for x in range(1, len(cfg_infor)-2):
			tmp_str = cfg_infor[x].strip()
			space_index = tmp_str.find(" ")
			dict_key = tmp_str[:space_index].strip()
			dict_value = tmp_str[space_index:].strip()
			dict_return[dict_key] = dict_value

		return data_type, dict_return
			

	def __get_dict_via_group_service(self, list_hostgroup, list_service):
		"""
		Get the full dict according the lists of group and services
		"""
		dict_return = {}

		# Get the dict key 'group_name', members and alias according to hostgroup list
		for group in list_hostgroup:
			tmp_dict={}
			group_name = group['hostgroup_name']
			tmp_members_list = group['members'].split(',')
			group_members_list = [ x.strip().split('_')[-1] for x in tmp_members_list]
			tmp_dict["ip"] = group_members_list
			tmp_dict["alias"] = group['alias']
			dict_return[group_name] = tmp_dict

		# Get the other information, eg. authlevel, userid, password and soon
		for service in list_service:
			tmp_dict = {}
			group_name = service['hostgroup_name']
			if group_name in dict_return:
				service_info = service['check_command'].split('!')
				if service_info[0] == "check_system_authPriv" and len(service_info) == 9 :
					tmp_dict['authlevel'] = "authPriv"
					tmp_dict['user'] = service_info[1]
					tmp_dict['snmpaprotocol'] = service_info[2]
					tmp_dict['apassword'] = service_info[3]
					tmp_dict['atag'] = service_info[4]
					tmp_dict['snmppprotocol'] = service_info[5]
					tmp_dict['ppassword'] = service_info[6]
					tmp_dict['ptag'] = service_info[7]
					tmp_dict['services'] = [service_info[8]]
				elif service_info[0] == "check_system_authNoPriv" and len(service_info) == 6 :
					tmp_dict['authlevel'] = "authnopriv"
					tmp_dict['user'] = service_info[1]
					tmp_dict['snmpaprotocol'] = service_info[2]
					tmp_dict['apassword'] = service_info[3]
					tmp_dict['atag'] = service_info[4]
					tmp_dict['services'] = [service_info[5]]
				elif service_info[0] == "check_system_noAuthNoPriv" and len(service_info) == 3 :
					tmp_dict['authlevel'] = "noauthnopriv"
					tmp_dict['user'] = service_info[1]
					tmp_dict['services'] = [service_info[2]]
				# This case is set for passive alert services, critical/non-critical/system/recovery
				elif service_info[0] == "check-host-alive":
					tmp_dict['services'] = ["passive_alert"]					

				dict_return[group_name] = self._merge_common_dict(dict_return[group_name], tmp_dict)
		
		return dict_return
	

	def save_config_file(self, dict_full, config_path):
		"""
		Save config file according to dict which include host, group and servers
		"""
		config_str = self.__dict2str(dict_full)
		return self.__write_config_file(config_str, config_path)

	
	def __dict2str(self, dict_full):
		"""
		Get the config file string according the dict information 
		"""
		host_cfg_str = ""
		group_cfg_str = ""
		services_cfg_str = ""
		for group, group_data in dict_full.items():
			xcc_ip_list, engineid, alias, user, authlevel, snmp_aprotocol, apassword, atag, snmp_pprotocol, ppassword, ptag, services_list = get_group_dict_info(group_data)
			if 0 == len(services_list):
				services_list = ['all', 'power', 'cpu', 'storage', 'temperature', 'fans', 'voltage', 'memory']
		
			## Get the host config
			for xcc_ip in xcc_ip_list:
				host_cfg_str += host_tmp % (group+'_'+xcc_ip, group+'_'+xcc_ip, xcc_ip)

			## Get the group config
			## Get the members of this group
			members_list = [group +"_" + xcc_ip for xcc_ip in xcc_ip_list]
			group_members = ','.join(members_list)
			if group_members:
				group_cfg_str += host_group_tmp % (group, alias, group_members)

			## Get the services config
			## get command for services
			if 'authpriv' == authlevel.lower():
				if '' == apassword or '' == ppassword:
					print("Both of authentication and privacy protocol password are necessary for authPriv level")
					continue
				else:
					if atag and ptag:
						pass
					else:
						apassword, atag = encrypt(apassword) 
						ppassword, ptag = encrypt(ppassword) 
					command_str = "%s!%s!%s!%s!%s!%s!%s!%s" % ("check_system_authPriv", user, snmp_aprotocol, apassword, atag, snmp_pprotocol, ppassword, ptag)
			elif 'authnopriv' == authlevel.lower():
				if '' == apassword:
					print("Authentication protocol password is necessary for authNoPriv level")
					continue
				else:
					if atag:
						pass
					else:
						apassword, atag = encrypt(apassword) 
					command_str = "%s!%s!%s!%s!%s" % ("check_system_authNoPriv", user, snmp_aprotocol, apassword, atag)
			elif 'noauthnopriv' == authlevel.lower():
				command_str = "%s!%s" % ("check_system_noAuthNoPriv", user)
			else:
				continue

			## add services 
			if services_list:
				services_cfg_str += ("\n#### Services for %s ####\n" % group)

			has_passive_alert = False
			for service in services_list:
				if "passive_alert" == service:
					has_passive_alert = True
					continue
				services_cfg_str += service_tmp % (group, service, command_str + "!" + service)
			if has_passive_alert:
				#alert_descript_list = ["Critical Alert", "Non-Critical Alert", "System Alert", "Recovery"]
				alert_descript_list = ["Critical Alert", "System Alert"]
				for alert_str in alert_descript_list:
					services_cfg_str += alert_service_tmp % (group, alert_str)

		return host_note + host_cfg_str + group_note + group_cfg_str + service_note + services_cfg_str


	def write_command_file(self, local_path):
		"""
		Write command.cfg to /opt/lenovo/nagios_lenovo_plugins/tools/etc/
		"""
		return self.__write_config_file(command_tmp, "%stools/etc/lenovo_snmp_command.cfg" % local_path)


	def __write_config_file(self, config_str, config_file):
		"""
		Write the config file to config file, config_file_name = path + file name
		"""
		try:
			fh = open(config_file, 'w+')
			fh.write(config_str)
			fh.close()
		except:
			print("Failed in writing configuration file")
			return False
		return True

	
	# Reboot nagios or cmk
	def __reboot_site(self):
		"""Restart Nagios or Check_MK with commands:
		For Nagios, run command 'service nagios restart' to restart 
		For Check_MK, run command 'cmk -R' to restart
		"""

		cmd = "service nagios restart"
		select_val = select_y_n("Do you want to restart Nagios? (y/n)(Blank is y)")

		if 'y' == select_val.lower():
			run_watch = subprocess.Popen(cmd.split(), stdout = subprocess.PIPE, stderr = subprocess.PIPE, universal_newlines=True)
			ret, err_msg = run_watch.communicate()
			printf ("Restart Messages:\n%s" % ret + err_msg)


	###############################################################
	#
	# Nagios plugin api for auto install
	#
	###############################################################

	def install_site(self, config_file, passive_cfg, importfile):
		nagios_path = self.site_path
		local_path = self.local_path
		actv_dict = dict()
		import_dict = dict()
		nagios_cfg_file = "%setc/lenovo_cfg/%s" % (nagios_path, NAGIOS_CFG_NAME)
		local_ngs_cfg_file = "%stools/etc/%s" % (local_path, NAGIOS_CFG_NAME)

		if self._check_privileges(self.site_path):
			pass
		else:
			printf ("\nPlease run this script with Nagios administrator privileges!\n")
			exit(0)

		self._clear_etc()

		self.__check_dependence()
		
		self.__install_plugin()

		run_adiscovery = select_y_n("Do you wish to run active discovery? (y/n) (Blank is y)")
		if 'y' == run_adiscovery:
			actv_dict = self.__get_actv_dscvr_dict(config_file)

		if importfile:
			import_dict = self.__get_import_dict(importfile)

		# Merge dicts from active discovery and import dict
		merged_dict = self.merge_dict(actv_dict, import_dict)

		load_dict = self.load_config_file(nagios_cfg_file)

		final_dict = self.merge_dict(load_dict, merged_dict)

		if final_dict:
			self.save_config_file(final_dict, local_ngs_cfg_file)
		
		self.write_command_file(self.local_path)

		self.__set_contacts_group()

		self.__install_etc_file(nagios_path)

		self.__modify_nagios_cfg(nagios_path)

		self.__verify_cfg(nagios_path)

		run_pdiscovery = select_y_n("Do you wish to run passive discovery? (y/n) (Blank is y)")
		if 'y' == run_pdiscovery:
			self.__passive_discovery(passive_cfg, "%s/etc/lenovo_cfg/%s" % (nagios_path, NAGIOS_CFG_NAME), nagios_path)

		self.__reboot_site()


	def active_discovery_site(self, config_file, importfile):

		nagios_path = self.site_path
		local_path = self.local_path
		nagios_cfg_file = "%setc/lenovo_cfg/%s" % (nagios_path, NAGIOS_CFG_NAME)
		local_ngs_cfg_file = "%stools/etc/%s" % (local_path, NAGIOS_CFG_NAME)
		actv_dict = dict()
		import_dict = dict()

		print("Active discovery is beginning...")

		actv_dict = self.__get_actv_dscvr_dict(config_file)

		if importfile:
			import_dict = self.__get_import_dict(importfile)

		# Merge dicts from active discovery and import dict
		merged_dict = self.merge_dict(actv_dict, import_dict)

		load_dict = self.load_config_file(nagios_cfg_file)

		final_dict = self.merge_dict(load_dict, merged_dict)

		self.save_config_file(final_dict, local_ngs_cfg_file)
		
		self.__install_etc_file(nagios_path)

		self.__verify_cfg(nagios_path)

		self.__reboot_site()
	


	def passive_discovery_site(self, passive_cfg):

		nagios_path = self.site_path

		print("Passive discovery is beginning...")

		self.write_command_file(self.local_path)

		self.__install_etc_file(nagios_path)

		self.__passive_discovery(passive_cfg, "%s/etc/lenovo_cfg/%s" % (nagios_path, NAGIOS_CFG_NAME), nagios_path)


if __name__ == "__main__":
	only_adiscovery = False
	only_pdiscovery = False
	config_file = ""
	nagios_path = ""
	importfile = ""
	passive_cfg_file = ""

	local_path = "../"
	h = NagiosPlugin(local_path, config_file, importfile, only_adiscovery, False)

	if h._check_privileges(h.site_path):
		pass
	else:
		printf ("\nPlease run this script with Nagios administrator privileges!\n")
		exit(0)

	if only_adiscovery:
		h.active_discovery_site(h.config_file, h.importfile)
	elif only_pdiscovery:
		exit(0)
	else:
		h.install_site(h.config_file, "", h.importfile)	
